

	*****************************
	*
	* HEA IDR Analysis
	*
	* Authors: CJ Libassi
	*
	* Date: 12.8.2017
	* 
	*****************************

	*Boilerplate
	clear all
	set more off 
	set maxvar 32000
	set seed 123456789
	set rmsg on 
	
	capture log close 
	
	*Set Globals
	global idr_calc = ""
	global output = ""
	
	****************************************************************************
	*
	*
	* Part 1: Calculate Pay-as-you earn output for each income & debt combo 
	*
	*
	*                               PAYE
	****************************************************************************
	
	*Step 0 - Input IDR Parameters
	
		local interest_rate = .05 //Decimal Between 0 and 1
		local repayment_length = 20
		local income_growth = .05  //Decimal Between 0 and 1
		local starting_fpl = 12060
		local fpl_ratio = 1.5 
		local fpl_growth = .02  //Decimal Between 0 and 1
		local disc_rate = .028  //Decimal Between 0 and 1
		local income_portion = .1  //Decimal Between 0 and 1
		local monthly_interest = `interest_rate'/12
	
		local start_income = 0
		local end_income = 100000
		local income_increment = 500
		
		local start_debt = 1000
		local end_debt = 65000
		local debt_increment = 500
		local plan_name "PAYE" // Whatever you want, but probably PAYE, REPAYE, or HEA
		
		
	*Set Globals
	*global idr_calc = "" // SET THE DIRECTORY WHERE YOU WOULD LIKE TO STORE DATA
	*global output = "" // SET THE DIRECTORY WHERE YOU WOULD LIKE TO STORE RESULTS
	
	
	*Set up Dataset 
		
		*Set local equal to number of income steps plus 1
		local inc_steps = 1+(`end_income'-`start_income')/`income_increment'
			
		*Set obs equal to number of income steps 
		set obs `inc_steps'
	
	*Fill in income 
	local count = 1 
		
	gen income = . 
		
	forvalues i = `start_income'(`income_increment')`end_income'{
		
		replace income = `i' if _n==`count'
		
		local count = 1+`count'
		
	}
		
	*Make Copies of Each of These Observations, one for every increment of debt 
		
		*Set local equal to number of debt steps plus 1
		local debt_steps = 1+(`end_debt'-`start_debt')/`debt_increment'
		
		expand `debt_steps'
		sort income 
		
	*Generate a Group ID 
	bys income: gen income_level_id = _n 
		
	*Fill in Debt 
	local count = 1 
		
		gen principal_balance = . 
		
		forvalues i = `start_debt'(`debt_increment')`end_debt'{
		
			replace principal_balance = `i' if income_level_id==`count'
		
			local count = 1+`count'
		
		}

	
	drop if principal_balance==. &  income==.
	
	*Step 1: Calculate 10-yr standard payment for each program
				* A = P*((r(1+r)^n)/((1+r)^n)-1)
				
	local monthly_interest = `interest_rate'/12
	
	gen standard_payment = principal_balance*((`monthly_interest'*(1+`monthly_interest')^120))/(((1+`monthly_interest')^120)-1)
		
		*Replace if payment isn't at least the minimum payment of $50
		replace standard_payment = 50 if standard_payment>0 & standard_payment<50 & !missing(standard_payment) 
			
		*Figure out the annual payment
		gen annual_payment = standard_payment*12 
			
			/*Replace annual payment with the principal balance if the minimum 
				annual payment (12*50=$600) is greater than original principal 
				balance	*/
			replace annual_payment = principal_balance if principal_balance<600 

	
	*Step 2: Calculate Discretionary Income in base year, to serve as basis for IDR payment calc
		
		*Earnings for Year 1 of Repayment
		gen earnings_year_1 = income 
		gen disc_med_earn_year_1 = .
		
			*Find 150% of the poverty line
			local fpl_150 = `starting_fpl'*`fpl_ratio'
			
				*Discretionary earnings should be zero if earnings are below 150% of the poverty line
				replace disc_med_earn_year_1 = 0 if income<=`fpl_150'
				*Otherwise discretionary earnings are income less 150% of the poverty line 
				replace disc_med_earn_year_1 = income - `fpl_150' if income > `fpl_150'
				
			gen fpl_150_year_1 = `fpl_150'
				
		*Set Earnings Appreciation Rate
		local earn_grow = `income_growth'
		local earn_grow_factor = 1+`income_growth'
		
		*Set Poverty Level Growth Rate
		local pov_grow_factor = 1+`fpl_growth'
		
		
	/*Step 3: Loop through going forward to calculate discretionary income,
			increasing income by and inflating poverty level every year */
		
		local pov_level = `starting_fpl' 
		
		forvalues i = 2/`repayment_length'{ 
		
			local pov_level = `pov_level'*`pov_grow_factor'
		
			local last_year= `i'-1
			
			gen earnings_year_`i' = earnings_year_`last_year'*`earn_grow_factor'
			gen disc_med_earn_year_`i' = .
			
				local fpl_150 = `pov_level'*`fpl_ratio'
				replace disc_med_earn_year_`i' = 0 if earnings_year_`i' <= `fpl_150'
				replace disc_med_earn_year_`i'= earnings_year_`i' - `fpl_150' if earnings_year_`i'> `fpl_150'
				
				gen fpl_150_year_`i' = `pov_level'
				
			}

			
	*Step 4: Determine if there is a partial financial hardship by comparing standard payment to 10% of Discretionary Income
		*Set blank variable
		gen interest_month_1_year_1 = 0
				
			*Generate interest acruals for 31 day months
			replace interest_month_1_year_1 = principal_balance*31*(`interest_rate'/365) ///
					if 1==1 | 1==3 | 1==5 | 1==7 | ///
							1==8 | 1==10 | 1==12  
								
			*Generate interest acruals for 30 day months			
			replace interest_month_1_year_1 = principal_balance*30*(`interest_rate'/365) ///
						if 1==4 | 1==6 | 1==9 | 1==11 
			
			*Generate interest acruals for February
			replace interest_month_1_year_1 = principal_balance*28*(`interest_rate'/365) ///
						if 1==2 

			*Add interest for this month to principal 
			egen amount_owed_1_year_1 = rowtotal(interest_month_1_year_1 principal_balance)
				
			*Calculate 10% of discretionary income, divide by 12
			gen ten_perc_1_year_1 = (disc_med_earn_year_1*`income_portion')/12
					
			*Calculate IDR Payment as minimum of ten percent and amount left to pay
			gen idr_payment_month_1_year_1 = min(ten_perc_1_year_1, amount_owed_1_year_1, standard_payment) 
				replace idr_payment_month_1_year_1=. if principal_balance==.
			
			*Calculate IDR Payment as max of amoritzed payment and $50, which is the min payment
			gen standard_paymennt_month_1_year_1 = max(50, standard_payment) 
				
				*Create a fix if the outstanding balance is less than $50
				replace standard_paymennt_month_1_year_1 = principal_balance if principal_balance<50
		
			*Create an indicator for partial financial hardship if IDR payment would be lower than standard payment
			gen part_fin_hard = idr_payment_month_1_year_1<standard_paymennt_month_1_year_1 ///
					& !missing(standard_paymennt_month_1_year_1) & !missing(principal_balance)  
			
			*Drop all of these variables so the loop doesn't get screwed up 
			drop interest_month_1_year_1 amount_owed_1_year_1 ten_perc_1_year_1 ///
					idr_payment_month_1_year_1 standard_paymennt_month_1_year_1 
			
			
	/*Step 5: With Loops 
	
			a) Calculate Pay-as-you-earn payment for each year 
			b) Calculate monthly interest accruals
			c) Calculate portion of actual payment that is interest & principal
			d) Calculate interest running accumulation across months 
			e) Calculate monthly principal paydown
	
	*/
		*Set up running calculation of the principal balance & accumulating interest
		gen rp_running_princ_bal = principal_balance 
		gen std_running_princ_bal = principal_balance 
		gen act_running_princ_bal = principal_balance 
	
		gen rp_accum_excess_interest = 0 
		gen std_accum_excess_interest = 0 
		gen act_accum_excess_interest = 0 
	
		gen years_idr_higher_std = 0 
	
		*Loop through years
		forvalues year = 1/`repayment_length'{ 
		
			*Loop through months
			forvalues month = 1/12{
		
				*Check Principal Balance at beginning of month
				gen rp_running_princ_bal_`month'_`year' = rp_running_princ_bal
				gen std_running_princ_bal_`month'_`year' = std_running_princ_bal
				gen act_running_princ_bal_`month'_`year' = act_running_princ_bal
				
				*A) Calculate Interest Amount Acrrued in This Month
				
				*IDR 
				*Set blank variable
				gen rp_interest_month_`month'_year_`year' = 0
				
					*Generate interest acruals for 31 day months
					replace rp_interest_month_`month'_year_`year' = rp_running_princ_bal*31*(`interest_rate'/365) ///
							if `month'==1 | `month'==3 | `month'==5 | `month'==7 | ///
								`month'==8 | `month'==10 | `month'==12  
								
					*Generate interest acruals for 30 day months			
					replace rp_interest_month_`month'_year_`year' = rp_running_princ_bal*30*(`interest_rate'/365) ///
							if `month'==4 | `month'==6 | `month'==9 | `month'==11 
			
					*Generate interest acruals for February
					replace rp_interest_month_`month'_year_`year' = rp_running_princ_bal*28*(`interest_rate'/365) ///
							if `month'==2 
							
				*STD 
				*Set blank variable
				gen std_interest_month_`month'_year_`year' = 0
				
					*Generate interest acruals for 31 day months
					replace std_interest_month_`month'_year_`year' = std_running_princ_bal*31*(`interest_rate'/365) ///
							if `month'==1 | `month'==3 | `month'==5 | `month'==7 | ///
								`month'==8 | `month'==10 | `month'==12  
								
					*Generate interest acruals for 30 day months			
					replace std_interest_month_`month'_year_`year' = std_running_princ_bal*30*(`interest_rate'/365) ///
							if `month'==4 | `month'==6 | `month'==9 | `month'==11 
			
					*Generate interest acruals for February
					replace std_interest_month_`month'_year_`year' = std_running_princ_bal*28*(`interest_rate'/365) ///
							if `month'==2 
							
				*Actual 
				*Set blank variable
				gen act_interest_month_`month'_year_`year' = 0
				
					*Generate interest acruals for 31 day months
					replace act_interest_month_`month'_year_`year' = act_running_princ_bal*31*(`interest_rate'/365) ///
							if `month'==1 | `month'==3 | `month'==5 | `month'==7 | ///
								`month'==8 | `month'==10 | `month'==12  
								
					*Generate interest acruals for 30 day months			
					replace act_interest_month_`month'_year_`year' = act_running_princ_bal*30*(`interest_rate'/365) ///
							if `month'==4 | `month'==6 | `month'==9 | `month'==11 
			
					*Generate interest acruals for February
					replace act_interest_month_`month'_year_`year' = act_running_princ_bal*28*(`interest_rate'/365) ///
							if `month'==2 
							
				*B) Adjust Accumulated Interest So All Unpaid Interest Is Lumped into One Amount that must be paid off before touching principal
				
					*IDR
					*Add this month's interest to the running accumulated interest 
					egen rp_accum_int_month_`month'_year_`year' = rowtotal(rp_accum_excess_interest rp_interest_month_`month'_year_`year')
					
					replace rp_accum_excess_interest = rp_accum_int_month_`month'_year_`year'
					
					*STD
					*Add this month's interest to the running accumulated interest 
					egen std_accum_int_month_`month'_year_`year' = rowtotal(std_accum_excess_interest std_interest_month_`month'_year_`year')
					
					replace std_accum_excess_interest = std_accum_int_month_`month'_year_`year'
					
					*Actual
					*Add this month's interest to the running accumulated interest 
					egen act_accum_int_month_`month'_year_`year' = rowtotal(act_accum_excess_interest act_interest_month_`month'_year_`year')
					
					replace act_accum_excess_interest = act_accum_int_month_`month'_year_`year'
					
				*C) Calculate a total amount left to be paid (principal & interest) that cannot be exceded with any payment
				
					*IDR
					egen rp_total_balance_`month'_`year' = rowtotal(rp_running_princ_bal rp_accum_excess_interest)
					
					*STD
					egen std_total_balance_`month'_`year' = rowtotal(std_running_princ_bal std_accum_excess_interest)
					

				*D) Calculate IDR Payment and Breakdown 
				
					*IDR amount = minimum of 10% of discretionary income in that year or total balance as of that month
				
					*Calculate 10% of discretionary income, divide by 12
					gen ten_perc_`month'_year_`year' = (disc_med_earn_year_`year'*`income_portion')/12
					
					*Calculate IDR Payment as minimum of ten percent and amount left to pay
					gen idr_payment_month_`month'_year_`year' = min(ten_perc_`month'_year_`year', rp_total_balance_`month'_`year', standard_payment)  
					
					*IDR payment breakdown:
						
					*First calculate interest part by taking the minimum of the outstanding interest and the IDR payment
					gen idr_int_pt_`month'_`year' = min(rp_accum_excess_interest, idr_payment_month_`month'_year_`year')
						
					*Then calculate principal part by calculating the remaining payment not going toward interest
					gen idr_minus_int_`month'_`year' = idr_payment_month_`month'_year_`year'-idr_int_pt_`month'_`year'
					gen idr_princ_pt_`month'_`year' = max(idr_minus_int_`month'_`year', 0)
					
					*Find out how much interest goes unpaid, if any
					
						*Subtract from IDR payment amount of interest unpaid right now 
						gen idr_test_temp = rp_accum_int_month_`month'_year_`year'-idr_payment_month_`month'_year_`year'
						
						*If interest is greater than IDR payment, interest unpaid is accumulated interest minus IDR payment
						gen int_unpaid_`month'_`year' = rp_accum_int_month_`month'_year_`year'-idr_payment_month_`month'_year_`year' if idr_test_temp>0 & !missing(idr_test_temp) 
						
							*Otherwise unpaid interest is zero
							replace int_unpaid_`month'_`year'=0 if idr_test_temp<=0 & !missing(idr_test_temp) 
							
							drop idr_test_temp
						
					*Determine how much of unpaid interest should be subtracted
						
						*In years 1-3 we subtract half of everything (this is a rough approximation that assumes half of loans are subsidized, other half is unsubsidized) 
						gen int_unpaid_`month'_`year'_adjustment = int_unpaid_`month'_`year'/2 if `year'<4
							
							*In years 4 and above we will subtract nothing 
							replace int_unpaid_`month'_`year'_adjustment = 0 if `year'>3
							
					*Now adjust the accumulated interest so that the correct portion of the unpaid part for this month gets removed 
						
						*First store of the accumulated interest after adjustment at this point
						gen post_adj_rp_acc_exc_int_`month'_`year' = rp_accum_excess_interest-int_unpaid_`month'_`year'_adjustment
					
						*Next, actually apply the adjustment 
						replace rp_accum_excess_interest = post_adj_rp_acc_exc_int_`month'_`year'  
						
					*Then calculate the net change in interest as interest accumulated less interested paid 
					gen rp_interest_change_`month'_`year' = -1*(idr_int_pt_`month'_`year')
					
													
				*E) Calculate Standard Payment and Breakdown 
					
					*Calculate final standard payment as minimum of standard payment and amount left to pay 
					gen std_payment_`month'_`year' = min(standard_payment, std_total_balance_`month'_`year') 
					
					*First calculate interest part by taking the minimum of the outstanding interest and the standard payment
					gen std_int_pt_`month'_`year' = min(std_interest_month_`month'_year_`year', std_payment_`month'_`year')
						
					*Then calculate principal part by calculating the remaining payment not going toward interest
					gen std_minus_int_`month'_`year' = std_payment_`month'_`year'-std_interest_month_`month'_year_`year'
					gen std_princ_pt_`month'_`year' = max(std_minus_int_`month'_`year', 0)
					
					*Then calculate the net change in interest as interest accumulated less interested paid (in standard situation, interest never goes unpaid)
					gen std_interest_change_`month'_`year' = -1*(std_int_pt_`month'_`year')
					

				*F) Draw down the prinicipal & interest balances separately for IDR, Standard & Actual 
				
					*IDR 
					
					*Principal draw down. Take principal amount and subtract out principal part of payment
					replace rp_running_princ_bal  = round(rp_running_princ_bal-idr_princ_pt_`month'_`year', .00001)
					
						*Store the principal balance after payment 
						gen rp_post_rprinc_`month'_`year' = rp_running_princ_bal 
					
					*Interest draw down. Take accumulated interest amount and add net interest change
					replace years_idr_higher_std = years_idr_higher_std+1 if standard_payment<=idr_payment_month_`month'_year_`year' & !missing(idr_payment_month_`month'_year_`year')
					gen years_idr_higher_std_`month'_`year' = years_idr_higher_std
					
					replace rp_accum_excess_interest = round(rp_accum_excess_interest+rp_interest_change_`month'_`year', .00001)
					
					replace rp_running_princ_bal  = rp_running_princ_bal+rp_accum_excess_interest if years_idr_higher_std==1
					replace rp_accum_excess_interest = 0 if years_idr_higher_std==1
					
						*Store the interest balance after payment 
						gen rp_post_aint_month_`month'_year_`year' = rp_accum_excess_interest
						
						
						
					*Standard
						
					*Principal draw down. Take principal amount and subtract out principal part of payment
					replace std_running_princ_bal  = std_running_princ_bal-std_princ_pt_`month'_`year'
					
						*Store the principal balance after payment 
						gen std_post_rprinc_`month'_`year' = std_running_princ_bal 
					
					*Interest draw down. Take accumulated interest amount and add net interest change
					replace std_accum_excess_interest = std_accum_excess_interest+std_interest_change_`month'_`year'
					
						*Store the interest balance after payment 
						gen std_post_aint_month_`month'_year_`year' = std_accum_excess_interest
					
				
			
			}
		}
		
		
	**********************************
	*
	*	Calculations 
	*
	**********************************
	
	* Repayment Period in Months
		
	*Loop through years 
	forvalues year = 1/`repayment_length'{ 
	
		
		*Loop through months
		forvalues month = 1/12{
			
			
			*Generate an indicator if running balance is greater than zero and not missing in each month of each year
			gen not_paid_dummy_idr_`month'_`year' = (rp_running_princ_bal_`month'_`year'>1 ///
				& !missing(rp_running_princ_bal_`month'_`year'))	
				
			
			gen not_paid_dummy_std_`month'_`year' = (std_running_princ_bal_`month'_`year'>0 ///
				& !missing(std_running_princ_bal_`month'_`year'))	
				
			
		}
				
	}
	
	
	*Count up indicators, which should be 1 for each month where balance is not zero (ie not paid off)
	egen repayment_period_months = rowtotal(not_paid_dummy_idr*) 
	egen repayment_pd_std = rowtotal(not_paid_dummy_std*) 	
	
	*Figure out when the last payment is 
	gen repay_pd_year = round(repayment_period_months/12) 
	gen repay_pd_month = mod(repayment_period_months, 12)
		
		*Fixe modulus weirdness
		replace repay_pd_month = 12 if repay_pd_month==0
		replace repay_pd_year = repay_pd_year-1 if repay_pd_month==0
	
	local rp_period_months = `repayment_length'*12 
		
	gen last_month_payment_idr = idr_payment_month_12_year_`repayment_length' if repayment_period_months==`rp_period_months'
	replace last_month_payment_idr = idr_payment_month_1_year_1 if repayment_period_months==1
		
	* Total Amount Paid on IDR (Nominal) 
	egen total_paid_idr_nom = rowtotal(idr_payment_month_*)
	egen total_paid_std_nom = rowtotal(std_payment_*)
	
	* Total Amount Paid on IDR (Discounted) 
	gen total_paid_idr_disc = 0 
	gen total_paid_std_disc = 0 
	
	gen tp_nom_after = 0 
	
	*Start a Month Counter
	local month_count = 1 
	
	*Loop through years
	forvalues year = 1/`repayment_length'{ 
		
		*Loop through months
		forvalues month = 1/12{
			
			*For IDR 
			gen disc_idr_pay_month_`month'_year_`year' = idr_payment_month_`month'_year_`year'/((1+(`disc_rate'/12))^`month_count')
		
				gen total_paid_idr_disc_asof_`month'_`year' = total_paid_idr_disc
				
			replace total_paid_idr_disc = total_paid_idr_disc + disc_idr_pay_month_`month'_year_`year'
				
				gen total_paid_idr_disc_after_`month'_`year' = total_paid_idr_disc			
			
			
			
				
			local month_count = `month_count'+1 
		
		}
			
	}
	
	*Keep just key variables for output
	keep income principal_balance total_paid_idr_disc total_paid_idr_nom /// 
		repayment_period_months total_paid_std_nom part_fin_hard
	
	rename total_paid_idr_disc `plan_name'_tp
	rename total_paid_idr_nom `plan_name'_tp_nom
	rename repayment_period_months `plan_name'_months
	rename total_paid_std_nom STD_tp_nom 
	rename part_fin_hard `plan_name'_pfh
	
	save "${idr_calc}`plan_name'_total_paid.dta", replace 
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	****************************************************************************
	*
	*
	* Part 1: Calculate REPAYE output for each income & debt combo 
	*
	*
	*                               REPAYE - Undergrad
	****************************************************************************
	clear 
	
	*Step 0 - Input IDR Parameters
	
		local interest_rate = .05 //Decimal Between 0 and 1
		local repayment_length = 20
		local income_growth = .05  //Decimal Between 0 and 1
		local starting_fpl = 12060
		local fpl_ratio = 1.5 
		local fpl_growth = .02  //Decimal Between 0 and 1
		local disc_rate = .028  //Decimal Between 0 and 1
		local income_portion = .1  //Decimal Between 0 and 1
		
		local start_income = 0
		local end_income = 100000
		local income_increment = 500
		
		local start_debt = 1000
		local end_debt = 65000
		local debt_increment = 500
		
		local level "Undergrad" // Grad or Undergrad 
		local plan_name "REPAYE" // Whatever you want, but probably PAYE, REPAYE or HEA
		
	*Set Globals
	*global idr_calc = "" // SET THE DIRECTORY WHERE YOU WOULD LIKE TO STORE DATA 
	*global output = "" // SET THE DIRECTORY WHERE YOU WOULD LIKE TO STORE RESULTS
	
	
	*Set up Dataset 
		
		*Set local equal to number of income steps plus 1
		local inc_steps = 1+(`end_income'-`start_income')/`income_increment'
			
		*Set obs equal to number of income steps 
		set obs `inc_steps'
	
	*Fill in income
	local count = 1 
		
	gen income = . 
		
	forvalues i = `start_income'(`income_increment')`end_income'{
		
		replace income = `i' if _n==`count'
		
		local count = 1+`count'
		
	}
		
	*Make Copies of Each of These Observations, one for every increment of debt 
		
		*Set local equal to number of debt steps plus 1
		local debt_steps = 1+(`end_debt'-`start_debt')/`debt_increment'
		
		expand `debt_steps'
		sort income 
		
	*Generate a Group ID 
	bys income: gen income_level_id = _n 
		
	*Fill in Debt 
	local count = 1 
		
		gen principal_balance = . 
		
		forvalues i = `start_debt'(`debt_increment')`end_debt'{
		
			replace principal_balance = `i' if income_level_id==`count'
		
			local count = 1+`count'
		
		}

	
	drop if principal_balance==. &  income==.
	
	*Step 1: Calculate 10-yr standard payment for each program
				* A = P*((r(1+r)^n)/((1+r)^n)-1)
				
	local monthly_interest = `interest_rate'/12
	
	gen standard_payment = principal_balance*((`monthly_interest'*(1+`monthly_interest')^120))/(((1+`monthly_interest')^120)-1)
		
		*Replace if payment isn't at least the minimum payment of $50
		replace standard_payment = 50 if standard_payment>0 & standard_payment<50 & !missing(standard_payment) 
			
		*Figure out the annual payment
		gen annual_payment = standard_payment*12 
			
			/*Replace annual payment with the principal balance if the minimum 
				annual payment (12*50=$600) is greater than original principal 
				balance	*/
			replace annual_payment = principal_balance if principal_balance<600 

	
	*Step 2: Calculate Discretionary Income in base year, to serve as basis for IDR payment calc
		
		*Earnings for Year 1 of Repayment
		gen earnings_year_1 = income 
		gen disc_med_earn_year_1 = .
		
			*Find 150% of the poverty line
			local fpl_150 = `starting_fpl'*`fpl_ratio'
			
				*Discretionary earnings should be zero if earnings are below 150% of the poverty line
				replace disc_med_earn_year_1 = 0 if income<=`fpl_150'
				*Otherwise discretionary earnings are income less 150% of the poverty line 
				replace disc_med_earn_year_1 = income - `fpl_150' if income > `fpl_150'
				
			gen fpl_150_year_1 = `fpl_150'
				
		*Set Earnings Appreciation Rate
		local earn_grow = `income_growth'
		local earn_grow_factor = 1+`income_growth'
		
		*Set Poverty Level Growth Rate
		local pov_grow_factor = 1+`fpl_growth'
		
		
	/*Step 3: Loop through going forward to calculate discretionary income,
			increasing income by and inflating poverty level every year */
		
		local pov_level = `starting_fpl' 
		
		forvalues i = 2/`repayment_length'{ 
		
			local pov_level = `pov_level'*`pov_grow_factor'
		
			local last_year= `i'-1
			
			gen earnings_year_`i' = earnings_year_`last_year'*`earn_grow_factor'
			gen disc_med_earn_year_`i' = .
		
				
				local fpl_150 = `pov_level'*`fpl_ratio'
				
				replace disc_med_earn_year_`i' = 0 if earnings_year_`i' <= `fpl_150'
				replace disc_med_earn_year_`i'= earnings_year_`i' - `fpl_150' if earnings_year_`i'> `fpl_150'
				
				gen fpl_150_year_`i' = `pov_level'
				
			}

			
	*Step 4: Determine if there is a partial financial hardship by comparing standard payment to 10% of Discretionary Income
		*Set blank variable
		gen interest_month_1_year_1 = 0
				
			*Generate interest acruals for 31 day months
			replace interest_month_1_year_1 = principal_balance*31*(`interest_rate'/365) ///
					if 1==1 | 1==3 | 1==5 | 1==7 | ///
							1==8 | 1==10 | 1==12  
								
			*Generate interest acruals for 30 day months			
			replace interest_month_1_year_1 = principal_balance*30*(`interest_rate'/365) ///
						if 1==4 | 1==6 | 1==9 | 1==11 
			
			*Generate interest acruals for February
			replace interest_month_1_year_1 = principal_balance*28*(`interest_rate'/365) ///
						if 1==2 

			*Add interest for this month to principal 
			egen amount_owed_1_year_1 = rowtotal(interest_month_1_year_1 principal_balance)
				
			*Calculate 10% of discretionary income, divide by 12
			gen ten_perc_1_year_1 = (disc_med_earn_year_1*`income_portion')/12
					
			*Calculate IDR Payment as minimum of ten percent and amount left to pay
			gen idr_payment_month_1_year_1 = min(ten_perc_1_year_1, amount_owed_1_year_1) 
				replace idr_payment_month_1_year_1=. if principal_balance==.
			
			*Calculate IDR Payment as max of amoritzed payment and $50, which is the min payment
			gen standard_paymennt_month_1_year_1 = max(50, standard_payment) 
				
				*Create a fix if the outstanding balance is less than $50
				replace standard_paymennt_month_1_year_1 = principal_balance if principal_balance<50
		
			*Create an indicator for partial financial hardship if IDR payment would be lower than standard payment
			gen part_fin_hard = idr_payment_month_1_year_1<standard_paymennt_month_1_year_1 ///
					& !missing(standard_paymennt_month_1_year_1) & !missing(principal_balance)  
			
			*Drop all of these variables so the loop doesn't get screwed up 
			drop interest_month_1_year_1 amount_owed_1_year_1 ten_perc_1_year_1 ///
					idr_payment_month_1_year_1 standard_paymennt_month_1_year_1 
			
			
	/*Step 5: With Loops 
	
			a) Calculate Pay-as-you-earn payment for each year 
			b) Calculate monthly interest accruals
			c) Calculate portion of actual payment that is interest & principal
			d) Calculate interest running accumulation across months 
			e) Calculate monthly principal paydown
	
	*/
		*Set up running calculation of the principal balance & accumulating interest
		gen rp_running_princ_bal = principal_balance 
		gen std_running_princ_bal = principal_balance 
		gen act_running_princ_bal = principal_balance 
	
		gen rp_accum_excess_interest = 0 
		gen std_accum_excess_interest = 0 
		gen act_accum_excess_interest = 0 
	
		*Loop through years
		forvalues year = 1/`repayment_length'{ 
		
			*Loop through months
			forvalues month = 1/12{
		
				*Check Principal Balance at beginning of month
				gen rp_running_princ_bal_`month'_`year' = rp_running_princ_bal
				gen std_running_princ_bal_`month'_`year' = std_running_princ_bal
				gen act_running_princ_bal_`month'_`year' = act_running_princ_bal
				
				*A) Calculate Interest Amount Acrrued in This Month
				
				*IDR 
				*Set blank variable
				gen rp_interest_month_`month'_year_`year' = 0
				
					*Generate interest acruals for 31 day months
					replace rp_interest_month_`month'_year_`year' = rp_running_princ_bal*31*(`interest_rate'/365) ///
							if `month'==1 | `month'==3 | `month'==5 | `month'==7 | ///
								`month'==8 | `month'==10 | `month'==12  
								
					*Generate interest acruals for 30 day months			
					replace rp_interest_month_`month'_year_`year' = rp_running_princ_bal*30*(`interest_rate'/365) ///
							if `month'==4 | `month'==6 | `month'==9 | `month'==11 
			
					*Generate interest acruals for February
					replace rp_interest_month_`month'_year_`year' = rp_running_princ_bal*28*(`interest_rate'/365) ///
							if `month'==2 
							
				*STD 
				*Set blank variable
				gen std_interest_month_`month'_year_`year' = 0
				
					*Generate interest acruals for 31 day months
					replace std_interest_month_`month'_year_`year' = std_running_princ_bal*31*(`interest_rate'/365) ///
							if `month'==1 | `month'==3 | `month'==5 | `month'==7 | ///
								`month'==8 | `month'==10 | `month'==12  
								
					*Generate interest acruals for 30 day months			
					replace std_interest_month_`month'_year_`year' = std_running_princ_bal*30*(`interest_rate'/365) ///
							if `month'==4 | `month'==6 | `month'==9 | `month'==11 
			
					*Generate interest acruals for February
					replace std_interest_month_`month'_year_`year' = std_running_princ_bal*28*(`interest_rate'/365) ///
							if `month'==2 
							
							
				*B) Adjust Accumulated Interest So All Unpaid Interest Is Lumped into One Amount that must be paid off before touching principal
				
					*IDR
					*Add this month's interest to the running accumulated interest 
					egen rp_accum_int_month_`month'_year_`year' = rowtotal(rp_accum_excess_interest rp_interest_month_`month'_year_`year')
					
					replace rp_accum_excess_interest = rp_accum_int_month_`month'_year_`year'
					
					*STD
					*Add this month's interest to the running accumulated interest 
					egen std_accum_int_month_`month'_year_`year' = rowtotal(std_accum_excess_interest std_interest_month_`month'_year_`year')
					
					replace std_accum_excess_interest = std_accum_int_month_`month'_year_`year'
					
				*C) Calculate a total amount left to be paid (principal & interest) that cannot be exceded with any payment
				
					*IDR
					egen rp_total_balance_`month'_`year' = rowtotal(rp_running_princ_bal rp_accum_excess_interest)
					
					*STD
					egen std_total_balance_`month'_`year' = rowtotal(std_running_princ_bal std_accum_excess_interest)
					

				*D) Calculate IDR Payment and Breakdown 
				
					*IDR amount = minimum of 10% of discretionary income in that year or total balance as of that month
				
					*Calculate 10% of discretionary income, divide by 12
					gen ten_perc_`month'_year_`year' = (disc_med_earn_year_`year'*`income_portion')/12
					
					*Calculate IDR Payment as minimum of ten percent and amount left to pay
					gen idr_payment_month_`month'_year_`year' = min(ten_perc_`month'_year_`year', rp_total_balance_`month'_`year')  
					
					*IDR payment breakdown:
						
					*First calculate interest part by taking the minimum of the outstanding interest and the IDR payment
					gen idr_int_pt_`month'_`year' = min(rp_accum_excess_interest, idr_payment_month_`month'_year_`year')
						
					*Then calculate principal part by calculating the remaining payment not going toward interest
					gen idr_minus_int_`month'_`year' = idr_payment_month_`month'_year_`year'-idr_int_pt_`month'_`year'
					gen idr_princ_pt_`month'_`year' = max(idr_minus_int_`month'_`year', 0)
					
					*Find out how much interest goes unpaid, if any
					
						*Subtract from IDR payment amount of interest unpaid right now 
						gen idr_test_temp = rp_accum_int_month_`month'_year_`year'-idr_payment_month_`month'_year_`year'
						
						*If interest is greater than IDR payment, interest unpaid is accumulated interest minus IDR payment
						gen int_unpaid_`month'_`year' = rp_accum_int_month_`month'_year_`year'-idr_payment_month_`month'_year_`year' if idr_test_temp>0 & !missing(idr_test_temp) 
						
							*Otherwise unpaid interest is zero
							replace int_unpaid_`month'_`year'=0 if idr_test_temp<=0 & !missing(idr_test_temp) 
							
							drop idr_test_temp
						
					*Determine how much of unpaid interest should be subtracted
						
						*In all years we subtract half 
						gen int_unpaid_`month'_`year'_adjustment = int_unpaid_`month'_`year'/2 
							
					*Now adjust the accumulated interest so that the correct portion of the unpaid part for this month gets removed 
						
						*First store of the accumulated interest after adjustment at this point
						gen post_adj_rp_acc_exc_int_`month'_`year' = rp_accum_excess_interest-int_unpaid_`month'_`year'_adjustment
					
						*Next, actually apply the adjustment 
						replace rp_accum_excess_interest = post_adj_rp_acc_exc_int_`month'_`year'  
						
					*Then calculate the net change in interest as interest accumulated less interested paid 
					gen rp_interest_change_`month'_`year' = -1*(idr_int_pt_`month'_`year')
							
				*E) Calculate Standard Payment and Breakdown 
					
					*Calculate final standard payment as minimum of standard payment and amount left to pay 
					gen std_payment_`month'_`year' = min(standard_payment, std_total_balance_`month'_`year') 
					
					*First calculate interest part by taking the minimum of the outstanding interest and the standard payment
					gen std_int_pt_`month'_`year' = min(std_interest_month_`month'_year_`year', std_payment_`month'_`year')
						
					*Then calculate principal part by calculating the remaining payment not going toward interest
					gen std_minus_int_`month'_`year' = std_payment_`month'_`year'-std_interest_month_`month'_year_`year'
					gen std_princ_pt_`month'_`year' = max(std_minus_int_`month'_`year', 0)
					
					*Then calculate the net change in interest as interest accumulated less interested paid (in standard situation, interest never goes unpaid)
					gen std_interest_change_`month'_`year' = -1*(std_int_pt_`month'_`year')
					
				*G) Draw down the prinicipal & interest balances separately for IDR, Standard & Actual 
				
					*IDR 
					
					*Principal draw down. Take principal amount and subtract out principal part of payment
					replace rp_running_princ_bal  = round(rp_running_princ_bal-idr_princ_pt_`month'_`year', .00001)
					
						*Store the principal balance after payment 
						gen rp_post_rprinc_`month'_`year' = rp_running_princ_bal 
					
					*Interest draw down. Take accumulated interest amount and add net interest change
					replace rp_accum_excess_interest = round(rp_accum_excess_interest+rp_interest_change_`month'_`year', .00001)
					
						*Store the interest balance after payment 
						gen rp_post_aint_month_`month'_year_`year' = rp_accum_excess_interest
						
					*Standard
						
					*Principal draw down. Take principal amount and subtract out principal part of payment
					replace std_running_princ_bal  = std_running_princ_bal-std_princ_pt_`month'_`year'
					
						*Store the principal balance after payment 
						gen std_post_rprinc_`month'_`year' = std_running_princ_bal 
					
					*Interest draw down. Take accumulated interest amount and add net interest change
					replace std_accum_excess_interest = std_accum_excess_interest+std_interest_change_`month'_`year'
					
						*Store the interest balance after payment 
						gen std_post_aint_month_`month'_year_`year' = std_accum_excess_interest
								
			
			}
		}
		
		
	**********************************
	*
	*	Calculations 
	*
	**********************************
	
	* Repayment Period in Months
		
	*Loop through years 
	forvalues year = 1/`repayment_length'{ 
	
		
		*Loop through months
		forvalues month = 1/12{
			
			
			*Generate an indicator if running balance is greater than zero and not missing in each month of each year
			gen not_paid_dummy_idr_`month'_`year' = (rp_running_princ_bal_`month'_`year'>1 ///
				& !missing(rp_running_princ_bal_`month'_`year'))				
			
		}
				
	}
	
	
	*Count up indicators, which should be 1 for each month where balance is not zero (ie not paid off)
	egen repayment_period_months = rowtotal(not_paid_dummy_idr*) 
		
	*Figure out when the last payment is 
	gen repay_pd_year = round(repayment_period_months/12) 
	gen repay_pd_month = mod(repayment_period_months, 12)
			
		replace repay_pd_month = 12 if repay_pd_month==0
		replace repay_pd_year = repay_pd_year-1 if repay_pd_month==0
	
	
	local rp_period_months = `repayment_length'*12 
		
	gen last_month_payment_idr = idr_payment_month_12_year_`repayment_length' if repayment_period_months==`rp_period_months'
	replace last_month_payment_idr = idr_payment_month_1_year_1 if repayment_period_months==1
		
	* Total Amount Paid on IDR (Nominal) 
	egen total_paid_idr_nom = rowtotal(idr_payment_month_*)
	
	* Total Amount Paid on IDR (Discounted) 
	gen total_paid_idr_disc = 0 
	
	*Start a Month Counter
	local month_count = 1 
	
	*Loop through years
	forvalues year = 1/`repayment_length'{ 
		
		*Loop through months
		forvalues month = 1/12{
			
			gen disc_idr_pay_month_`month'_year_`year' = idr_payment_month_`month'_year_`year'/((1+(`disc_rate'/12))^`month_count')
		
				gen total_paid_idr_disc_asof_`month'_`year' = total_paid_idr_disc
				
			replace total_paid_idr_disc = total_paid_idr_disc + disc_idr_pay_month_`month'_year_`year'
				
				gen total_paid_idr_disc_after_`month'_`year' = total_paid_idr_disc
				
			
			
				
			local month_count = `month_count'+1 
		
		}
			
	}
	

	*Keep just key variables for output
	keep income principal_balance total_paid_idr_disc total_paid_idr_nom ///
		repayment_period_months part_fin_hard
	
	rename total_paid_idr_disc `plan_name'_`level'_tp
	rename total_paid_idr_nom `plan_name'_`level'_tp_nom
	rename repayment_period_months `plan_name'_`level'_months
	rename part_fin_hard `plan_name'_pfh
	
	save "${idr_calc}`plan_name'_total_paid.dta", replace 
	
	
	
	
	
	
	
	
	
	
	

	
	
	
	
	
	
	
	
	
	****************************************************************************/
	*
	*
	* Part 1: Calculate IDR output for each income & debt combo 
	*
	*
	*                         HEA Plan
	****************************************************************************
	clear 
	
	*Step 0 - Input IDR Parameters
	
		local interest_rate = .05 //Decimal Between 0 and 1
		local repayment_length = 75
		local income_growth = .05  //Decimal Between 0 and 1
		local starting_fpl = 12060
		local fpl_ratio = 1.5 
		local fpl_growth = .02  //Decimal Between 0 and 1
		local disc_rate = .028  //Decimal Between 0 and 1
		local income_portion = .15  //Decimal Between 0 and 1
		
		local start_income = 0
		local end_income = 100000
		local income_increment = 500
		
		local start_debt = 1000
		local end_debt = 65000
		local debt_increment = 500
		
		local plan_name "HEA" // Whatever you want, but probably PAYE, REPAYE, or TrIDR
		
	*Set Globals
	*global idr_calc = "" // SET THE DIRECTORY WHERE YOU WOULD LIKE TO STORE DATA
	*global output = "" // SET THE DIRECTORY WHERE YOU WOULD LIKE TO STORE RESULTS
	
	
	*Set up Dataset 
		
		*Set local equal to number of income steps plus 1
		local inc_steps = 1+(`end_income'-`start_income')/`income_increment'
			
		*Set obs equal to number of income steps 
		set obs `inc_steps'
	
	*Fill in income 
	local count = 1 
		
	gen income = . 
		
	forvalues i = `start_income'(`income_increment')`end_income'{
		
		replace income = `i' if _n==`count'
		
		local count = 1+`count'
		
	}
		
	*Make Copies of Each of These Observations, one for every increment of debt 
		
		*Set local equal to number of debt steps plus 1
		local debt_steps = 1+(`end_debt'-`start_debt')/`debt_increment'
		
		expand `debt_steps'
		sort income 
		
	*Generate a Group ID 
	bys income: gen income_level_id = _n 
		
	*Fill in Debt
	local count = 1 
		
		gen principal_balance = . 
		
		forvalues i = `start_debt'(`debt_increment')`end_debt'{
		
			replace principal_balance = `i' if income_level_id==`count'
		
			local count = 1+`count'
		
		}

	
	
	drop if principal_balance==. &  income==.
	
	*Step 1: Calculate 10-yr standard payment for each program
				* A = P*((r(1+r)^n)/((1+r)^n)-1)
				
	local monthly_interest = `interest_rate'/12
	
	gen standard_payment = principal_balance*((`monthly_interest'*(1+`monthly_interest')^120))/(((1+`monthly_interest')^120)-1)
		
		*Replace if payment isn't at least the minimum payment of $50
		replace standard_payment = 50 if standard_payment>0 & standard_payment<50 & !missing(standard_payment) 
			
		*Figure out the annual payment
		gen annual_payment = standard_payment*12 
			
			/*Replace annual payment with the principal balance if the minimum 
				annual payment (12*50=$600) is greater than original principal 
				balance	*/
			replace annual_payment = principal_balance if principal_balance<600 
			
		
	
	*Step 2: Calculate Discretionary Income in base year, to serve as basis for IDR payment calc
	
		*Earnings for Year 1 of Repayment
		gen earnings_year_1 = income 
		gen disc_med_earn_year_1 = .
		
			*Find 150% of the poverty line
			local fpl_150 = `starting_fpl'*`fpl_ratio'
			
				*Discretionary earnings should be zero if earnings are below 150% of the poverty line
				replace disc_med_earn_year_1 = 0 if income<=`fpl_150'
				*Otherwise discretionary earnings are income less 150% of the poverty line 
				replace disc_med_earn_year_1 = income - `fpl_150' if income > `fpl_150'
				
			gen fpl_150_year_1 = `fpl_150'
				
		*Set Earnings Appreciation Rate
		local earn_grow = `income_growth'
		local earn_grow_factor = 1+`income_growth'
		
		*Set Poverty Level Growth Rate
		local pov_grow_factor = 1+`fpl_growth'
		
		
	/*Step 3: Loop through going forward to calculate discretionary income,
			increasing income by and inflating poverty level every year */
		
		local pov_level = `starting_fpl' 
		
		forvalues i = 2/`repayment_length'{ 
		
			local pov_level = `pov_level'*`pov_grow_factor'
			
				di "Poverty level is `pov_level'" 
		
			local last_year= `i'-1
			
			gen earnings_year_`i' = earnings_year_`last_year'*`earn_grow_factor'
			gen disc_med_earn_year_`i' = .
			
				local fpl_150 = `pov_level'*`fpl_ratio'
				replace disc_med_earn_year_`i' = 0 if earnings_year_`i' <= `fpl_150'
				replace disc_med_earn_year_`i'= earnings_year_`i' - `fpl_150' if earnings_year_`i'> `fpl_150'
				
				gen fpl_150_year_`i' = `pov_level'
				
			}

			
	*Step 4: Determine if there is a partial financial hardship by comparing standard payment to 10% of Discretionary Income
		*Set blank variable
		gen interest_month_1_year_1 = 0
				
			*Generate interest acruals for 31 day months
			replace interest_month_1_year_1 = principal_balance*31*(`interest_rate'/365) ///
					if 1==1 | 1==3 | 1==5 | 1==7 | ///
							1==8 | 1==10 | 1==12  
								
			*Generate interest acruals for 30 day months			
			replace interest_month_1_year_1 = principal_balance*30*(`interest_rate'/365) ///
						if 1==4 | 1==6 | 1==9 | 1==11 
			
			*Generate interest acruals for February
			replace interest_month_1_year_1 = principal_balance*28*(`interest_rate'/365) ///
						if 1==2 

			*Add interest for this month to principal 
			egen amount_owed_1_year_1 = rowtotal(interest_month_1_year_1 principal_balance)
				
			*Calculate 10% of discretionary income, divide by 12
			gen ten_perc_1_year_1 = (disc_med_earn_year_1*`income_portion')/12
					
			*Calculate IDR Payment as minimum of ten percent and amount left to pay
			gen idr_payment_month_1_year_1 = min(ten_perc_1_year_1, amount_owed_1_year_1) 
				replace idr_payment_month_1_year_1=. if principal_balance==.
			
			*Calculate IDR Payment as max of amoritzed payment and $50, which is the min payment
			gen standard_paymennt_month_1_year_1 = max(50, standard_payment) 
				
				*Create a fix if the outstanding balance is less than $50
				replace standard_paymennt_month_1_year_1 = principal_balance if principal_balance<50
		
			*Create an indicator for partial financial hardship if IDR payment would be lower than standard payment
			gen part_fin_hard = idr_payment_month_1_year_1<standard_paymennt_month_1_year_1 ///
					& !missing(standard_paymennt_month_1_year_1) & !missing(principal_balance)  
			
			*Drop all of these variables so the loop doesn't get screwed up 
			drop interest_month_1_year_1 amount_owed_1_year_1 ten_perc_1_year_1 ///
					idr_payment_month_1_year_1 standard_paymennt_month_1_year_1 
			
			
	/*Step 5: With Loops 
	
			a) Calculate Pay-as-you-earn payment for each year 
			b) Calculate monthly interest accruals
			c) Calculate portion of actual payment that is interest & principal
			d) Calculate interest running accumulation across months 
			e) Calculate monthly principal paydown
	
	*/
		*Set up running calculation of the principal balance & accumulating interest
		gen std_running_princ_bal = principal_balance 
		gen std_accum_excess_interest = 0 
	
		*Loop through years
		forvalues year = 1/10{ 
		
			*Loop through months
			forvalues month = 1/12{	
					
				*Check Principal Balance at beginning of month
				gen std_running_princ_bal_`month'_`year' = std_running_princ_bal
				
				*A) Calculate Interest Amount Acrrued in This Month			
				
				*STD 
				*Set blank variable
				gen std_interest_month_`month'_year_`year' = 0
				
					*Generate interest acruals for 31 day months
					replace std_interest_month_`month'_year_`year' = std_running_princ_bal*31*(`interest_rate'/365) ///
							if `month'==1 | `month'==3 | `month'==5 | `month'==7 | ///
								`month'==8 | `month'==10 | `month'==12  
								
					*Generate interest acruals for 30 day months			
					replace std_interest_month_`month'_year_`year' = std_running_princ_bal*30*(`interest_rate'/365) ///
							if `month'==4 | `month'==6 | `month'==9 | `month'==11 
			
					*Generate interest acruals for February
					replace std_interest_month_`month'_year_`year' = std_running_princ_bal*28*(`interest_rate'/365) ///
							if `month'==2 
							
							
				*B) Adjust Accumulated Interest So All Unpaid Interest Is Lumped into One Amount that must be paid off before touching principal
									
					*STD
					*Add this month's interest to the running accumulated interest 
					egen std_accum_int_month_`month'_year_`year' = rowtotal(std_accum_excess_interest std_interest_month_`month'_year_`year')
					
					replace std_accum_excess_interest = std_accum_int_month_`month'_year_`year'
					
				*C) Calculate a total amount left to be paid (principal & interest) that cannot be exceded with any payment
				
					*STD
					egen std_total_balance_`month'_`year' = rowtotal(std_running_princ_bal std_accum_excess_interest)
					
							
				*E) Calculate Standard Payment and Breakdown 
					
					*Calculate final standard payment as minimum of standard payment and amount left to pay 
					gen std_payment_`month'_`year' = min(standard_payment, std_total_balance_`month'_`year') 
					
					*First calculate interest part by taking the minimum of the outstanding interest and the standard payment
					gen std_int_pt_`month'_`year' = min(std_interest_month_`month'_year_`year', std_payment_`month'_`year')
						
					*Then calculate principal part by calculating the remaining payment not going toward interest
					gen std_minus_int_`month'_`year' = std_payment_`month'_`year'-std_interest_month_`month'_year_`year'
					gen std_princ_pt_`month'_`year' = max(std_minus_int_`month'_`year', 0)
					
					*Then calculate the net change in interest as interest accumulated less interested paid (in standard situation, interest never goes unpaid)
					gen std_interest_change_`month'_`year' = -1*(std_int_pt_`month'_`year')
					
				
				*G) Draw down the prinicipal & interest balances separately for IDR, Standard & Actual 
				
					*Standard
						
					*Principal draw down. Take principal amount and subtract out principal part of payment
					replace std_running_princ_bal  = std_running_princ_bal-std_princ_pt_`month'_`year'
					
						*Store the principal balance after payment 
						gen std_post_rprinc_`month'_`year' = std_running_princ_bal 
					
					*Interest draw down. Take accumulated interest amount and add net interest change
					replace std_accum_excess_interest = std_accum_excess_interest+std_interest_change_`month'_`year'
					
						*Store the interest balance after payment 
						gen std_post_aint_month_`month'_year_`year' = std_accum_excess_interest				
			
			}
		}
		
	
	*Calculate total amount paid on Standard Plan
	egen total_paid_std = rowtotal(std_payment_*)
	
	drop std_running_princ_bal-std_post_aint_month_12_year_10
	
	*Set up running calculation of the principal balance & accumulating interest
	gen rp_running_princ_bal = principal_balance 
	gen rp_accum_excess_interest = 0 
	
	
	*Calculate the total amount paid as of each month to be able to check against standard amount
	
	*Loop through years
	forvalues year = 1/`repayment_length'{ 
		
		*Loop through months
		forvalues month = 1/12{
		
			gen total_paid_as_of_`month'_`year' = 0 if `year'==1 & `month'==1
			
			if `year'>1 | `month'>1{
			
				egen total_paid_as_of_`month'_`year'_help = rowtotal(idr_payment_month_*)
				
				replace total_paid_as_of_`month'_`year' = total_paid_as_of_`month'_`year'_help 
			
			}
			
		
			*Check Principal Balance at beginning of month
			gen rp_running_princ_bal_`month'_`year' = rp_running_princ_bal
				
			*A) Calculate Interest Amount Acrrued in This Month
				
			*IDR 
				
			*Set blank variable
			gen rp_interest_month_`month'_year_`year' = 0
			
			gen paying_`month'_`year' = 1 if (total_paid_std>total_paid_as_of_`month'_`year' & !missing(total_paid_std)) & rp_running_princ_bal>0 & !missing(rp_running_princ_bal) 
				replace paying_`month'_`year' = 0 if (total_paid_std<total_paid_as_of_`month'_`year' & !missing(total_paid_as_of_`month'_`year')) & rp_running_princ_bal>0  & !missing(rp_running_princ_bal)
					
			*Generate interest acruals for 31 day months
			replace rp_interest_month_`month'_year_`year' = rp_running_princ_bal*31*(`interest_rate'/365) ///
						if (total_paid_std>total_paid_as_of_`month'_`year' & !missing(total_paid_std)) & ///
						(`month'==1 | `month'==3 | `month'==5 | `month'==7 | ///
							`month'==8 | `month'==10 | `month'==12 ) 
									
			*Generate interest acruals for 30 day months			
			replace rp_interest_month_`month'_year_`year' = rp_running_princ_bal*30*(`interest_rate'/365) ///
						if (total_paid_std>total_paid_as_of_`month'_`year' & !missing(total_paid_std)) & ///
						(`month'==4 | `month'==6 | `month'==9 | `month'==11) 
				
			*Generate interest acruals for February
			replace rp_interest_month_`month'_year_`year' = rp_running_princ_bal*28*(`interest_rate'/365) ///
						if (total_paid_std>total_paid_as_of_`month'_`year' & !missing(total_paid_std)) & `month'==2 
						
			
						
			*B) Adjust Accumulated Interest So All Unpaid Interest Is Lumped into One Amount that must be paid off before touching principal
			
				*IDR
				*Add this month's interest to the running accumulated interest 
				egen rp_accum_int_month_`month'_year_`year' = rowtotal(rp_accum_excess_interest rp_interest_month_`month'_year_`year')
				
				replace rp_accum_excess_interest = rp_accum_int_month_`month'_year_`year'
						
				
			*C) Calculate a total amount left to be paid (principal & interest) that cannot be exceded with any payment
				
					*IDR
					egen rp_total_balance_`month'_`year' = rowtotal(rp_running_princ_bal rp_accum_excess_interest)
				
				
			*D) Calculate IDR Payment and Breakdown 
				
					*IDR amount = minimum of 10% of discretionary income in that year or total balance as of that month
				
					*Calculate 10% of discretionary income, divide by 12
					gen ten_perc_`month'_year_`year' = (disc_med_earn_year_`year'*`income_portion')/12
					
					*Calculate IDR Payment as minimum of ten percent and amount left to pay
					gen t1_idr_payment_month_`month'_year_`year' = max(ten_perc_`month'_year_`year', 25) 
					gen idr_payment_month_`month'_year_`year' = min(t1_idr_payment_month_`month'_year_`year', rp_total_balance_`month'_`year')    
						replace idr_payment_month_`month'_year_`year' = 0 if paying_`month'_`year'==0
					
					*IDR payment breakdown:
						
					*First calculate interest part by taking the minimum of the outstanding interest and the IDR payment
					gen idr_int_pt_`month'_`year' = min(rp_accum_excess_interest, idr_payment_month_`month'_year_`year')
						
					*Then calculate principal part by calculating the remaining payment not going toward interest
					gen idr_minus_int_`month'_`year' = idr_payment_month_`month'_year_`year'-idr_int_pt_`month'_`year'
					gen idr_princ_pt_`month'_`year' = max(idr_minus_int_`month'_`year', 0)
					
					*Find out how much interest goes unpaid, if any
					
						*Subtract from IDR payment amount of interest unpaid right now 
						gen idr_test_temp = rp_accum_int_month_`month'_year_`year'-idr_payment_month_`month'_year_`year'
						
						*If interest is greater than IDR payment, interest unpaid is accumulated interest minus IDR payment
						gen int_unpaid_`month'_`year' = rp_accum_int_month_`month'_year_`year'-idr_payment_month_`month'_year_`year' if idr_test_temp>0 & !missing(idr_test_temp) 
						
							*Otherwise unpaid interest is zero
							replace int_unpaid_`month'_`year'=0 if idr_test_temp<=0 & !missing(idr_test_temp) 
							
							drop idr_test_temp
						
					*Then calculate the net change in interest as interest accumulated less interested paid 
					gen rp_interest_change_`month'_`year' = -1*(idr_int_pt_`month'_`year')
				
							
			*G) Draw down the prinicipal & interest balances separately for IDR, Standard & Actual 
				
				*IDR 
					
				*Principal draw down. Take principal amount and subtract out principal part of payment
				replace rp_running_princ_bal  = round(rp_running_princ_bal-idr_princ_pt_`month'_`year', .00001)
					
					*Store the principal balance after payment 
					gen rp_post_rprinc_`month'_`year' = rp_running_princ_bal 
					
				*Interest draw down. Take accumulated interest amount and add net interest change
				replace rp_accum_excess_interest = round(rp_accum_excess_interest+rp_interest_change_`month'_`year', .00001)
					
					*Store the interest balance after payment 
					gen rp_post_aint_month_`month'_year_`year' = rp_accum_excess_interest
											
		
				
			}
		}
		
		
		
	**********************************
	*
	*	Calculations 
	*
	**********************************
	
	* Repayment Period in Months
		
	*Loop through years 
	forvalues year = 1/`repayment_length'{ 
	
		
		*Loop through months
		forvalues month = 1/12{
			
			
			*Generate an indicator if running balance is greater than zero and not missing in each month of each year
			gen not_paid_dummy_idr_`month'_`year' = (rp_running_princ_bal_`month'_`year'>1 ///
				& !missing(rp_running_princ_bal_`month'_`year'))				
			
		}
				
	}
	
	
	*Count up indicators, which should be 1 for each month where balance is not zero (ie not paid off)
	egen repayment_period_months = rowtotal(paying*) 
		
	*Figure out when the last payment is 
	gen repay_pd_year = round(repayment_period_months/12) 
	gen repay_pd_month = mod(repayment_period_months, 12)
			
		replace repay_pd_month = 12 if repay_pd_month==0
		replace repay_pd_year = repay_pd_year-1 if repay_pd_month==0
	
	local rp_period_months = `repayment_length'*12 
		
	* Total Amount Paid on IDR (Nominal) 
	egen total_paid_idr_nom = rowtotal(idr_payment_month_*)
	
	* Total Amount Paid on IDR (Discounted) 
	gen total_paid_idr_disc = 0 
	
	*Start a Month Counter
	local month_count = 1 
	
	*Loop through years
	forvalues year = 1/`repayment_length'{ 
		
		*Loop through months
		forvalues month = 1/12{
			
			gen disc_idr_pay_month_`month'_year_`year' = idr_payment_month_`month'_year_`year'/((1+(`disc_rate'/12))^`month_count')
		
				gen total_paid_idr_disc_asof_`month'_`year' = total_paid_idr_disc
				
			replace total_paid_idr_disc = total_paid_idr_disc + disc_idr_pay_month_`month'_year_`year'
				
				gen total_paid_idr_disc_after_`month'_`year' = total_paid_idr_disc
				
			
			
				
			local month_count = `month_count'+1 
		
		}
			
	}
	

	egen months_paying = rowtotal(paying_*) 
	
	*Keep just key variables for output
	keep income principal_balance total_paid_idr_disc total_paid_idr_nom ///
		repayment_period_months months_paying total_paid_std part_fin_hard
	
	rename total_paid_idr_disc `plan_name'_tp
	rename total_paid_idr_nom `plan_name'_tp_nom
	rename repayment_period_months `plan_name'_months
	rename part_fin_hard `plan_name'_pfh
	
	
	save "${idr_calc}`plan_name'_total_paid.dta", replace 
	

	
	

	
	
	
	
	
	
	
	
	
	
	***************************************************************************
	*
	*
	*   Part 2: Create one dataset with all the main findings from the individual plan analysis
	*
	*
	***************************************************************************
	
	
	
	*Create a single file of incomes, principals, and plan differences
	use "${idr_calc}PAYE_total_paid.dta", clear 
	
	merge 1:1 income principal_balance using "${idr_calc}REPAYE_total_paid.dta" 
		drop _merge
	
	merge 1:1 income principal_balance using "${idr_calc}HEA_total_paid.dta" 
		drop _merge
	
	keep if principal_balance<66000
	
	*Calculate difference from PAYE and REPAYE in total paid (discounted)
	gen hea_minus_paye_tp = HEA_tp-PAYE_tp
	gen hea_minus_repaye_tp = HEA_tp-REPAYE_Undergrad_tp
	
	*Calculate difference from PAYE and REPAYE in total paid (nominal)
	gen hea_minus_paye_tp_nom = HEA_tp_nom-PAYE_tp_nom
	gen hea_minus_repaye_tp_nom = HEA_tp_nom-REPAYE_Undergrad_tp_nom
	
	*Calculate difference from PAYE and REPAYE in length of repayment in months
	gen hea_minus_paye_months = HEA_months-PAYE_months 
	gen hea_minus_repaye_months = HEA_months-REPAYE_Undergrad_months 
	
	*Graph the Differences in Amount Paid
	twoway contour hea_minus_paye_tp principal_balance income if !missing(HEA_tp) & HEA_pfh==1, ///
		ccuts(-15000(15000)45000) scheme(plottig) clegend(position(3)) ///
		ccolors("102,189,99" "166,217,106" "217,239,139" "254,224,139" "253,174,97" "244,109,67") ///
		interp(none) ytitle("Principal Balance") xtitle("Income Entering Repayment") ///
		zlabel(,labsize(2.85)) ylabel(20000 "$20,000" 40000 "$40,000" 60000 "$60,000") ///
		xlabel(15000 "$15,000" 30000 "$30,000" 45000 "$45,000" 60000 "$60,000") ///
		ztitle("") title("Difference Between Amount Paid on HEA Plan and PAYE") ///
		subtitle("Net Present Value of Projected Total Payments on HEA Minus Those Made on PAYE", size(vsmall)) ///
		zlabel(-15000 "$-15,000" 0 "$0" 15000 "$15,000" 30000 "$30,000" 45000 "$45,000")	
		
		graph export "${output}difference_paid_from_current_law_paye.png", replace 
		
	putexcel set "${output}idr_analysis_results.xlsx", sheet("Amount Repaid") replace 
	
	putexcel A1 = picture("${output}difference_paid_from_current_law_paye.png")	
		
		
	twoway contour hea_minus_repaye_tp principal_balance income if !missing(HEA_tp) & HEA_pfh==1, ///
		ccuts(-15000(15000)45000) scheme(plottig) clegend(position(3)) ///
		ccolors("102,189,99" "166,217,106" "217,239,139" "254,224,139" "253,174,97" "244,109,67") ///
		interp(none) ytitle("Principal Balance") xtitle("Income Entering Repayment") ///
		zlabel(,labsize(2.85)) ylabel(20000 "$20,000" 40000 "$40,000" 60000 "$60,000") ///
		xlabel(15000 "$15,000" 30000 "$30,000" 45000 "$45,000" 60000 "$60,000") ///
		ztitle("") title("Difference Between Amount Paid on HEA Plan and REPAYE") ///
		subtitle("Net Present Value of Projected Total Payments on HEA Minus Those Made on REPAYE", size(vsmall)) ///
		zlabel(-15000 "$-15,000" 0 "$0" 15000 "$15,000" 30000 "$30,000" 45000 "$45,000")
		
		graph export "${output}difference_paid_from_current_law_repaye.png", replace 
		
	putexcel set "${output}idr_analysis_results.xlsx", sheet("Amount Repaid") modify 
	
	putexcel N1 = picture("${output}difference_paid_from_current_law_repaye.png")	
	
	*Graph the Differences in Time Repaying
	twoway contour hea_minus_paye_months principal_balance income if !missing(HEA_tp_nom) & HEA_pfh==1, ///
		scheme(plottig) clegend(position(3)) ccuts(-120(120)840) interp(none) ///
		xtitle("Income Entering Repayment") ///
		ccolors("158,1,66" "213,62,79" "244,109,67" "253,174,97" ///
		"254,224,139" "255,255,191" "230,245,152" "171,221,164" "102,194,165") ///
		ytitle("Principal Balance") ylabel(20000 "$20,000" 40000 "$40,000" 60000 "$60,000") ///
		xlabel(15000 "$15,000" 30000 "$30,000" 45000 "$45,000" 60000 "$60,000") ///
		ztitle("") zlabel(-120 "-10 Years" 0 "0 Years" 120 "10 Years" 240 "20 Years" 360 "30 Years" ///
		480 "40 Years" 600 "50 Years" 720 "60 Years" 840 "70 Years") ///
		ztitle("") title("Difference Between Years Repaying on HEA Plan and PAYE") ///
		subtitle("For Each Income & Debt Combination", size(small)) 
		
		graph export "${output}Dif_years_in_repayment_paye.png", replace  
		
	putexcel set "${output}idr_analysis_results.xlsx", sheet("Years Paying") modify 
	
	putexcel A1 = picture("${output}Dif_years_in_repayment_paye.png")
	
	twoway contour hea_minus_repaye_months principal_balance income if !missing(HEA_tp_nom) & HEA_pfh==1, ///
		scheme(plottig) clegend(position(3)) ccuts(-120(120)840) interp(none) ///
		xtitle("Income Entering Repayment") ///
		ccolors("158,1,66" "213,62,79" "244,109,67" "253,174,97" ///
		"254,224,139" "255,255,191" "230,245,152" "171,221,164" "102,194,165") ///
		ytitle("Principal Balance") ylabel(20000 "$20,000" 40000 "$40,000" 60000 "$60,000") ///
		xlabel(15000 "$15,000" 30000 "$30,000" 45000 "$45,000" 60000 "$60,000") ///
		ztitle("") zlabel(-120 "-10 Years" 0 "0 Years" 120 "10 Years" 240 "20 Years" 360 "30 Years" ///
		480 "40 Years" 600 "50 Years" 720 "60 Years" 840 "70 Years") ///
		title("Difference Between Years Repaying on HEA Plan and REPAYE") ///
		subtitle("For Each Income & Debt Combination", size(small)) 
		
		graph export "${output}Dif_years_in_repayment_repaye.png", replace
	
	putexcel N1 = picture("${output}Dif_years_in_repayment_repaye.png")
		
	
	
	
	***********************
	*
	* Export Everything to Excel 
	*
	***********************
	
	order income principal_balance STD_tp_nom PAYE_months PAYE_tp_nom PAYE_tp ///
		REPAYE_Undergrad_months REPAYE_Undergrad_tp_nom REPAYE_Undergrad_tp ///
		HEA_months HEA_tp_nom HEA_tp
		
	keep income principal_balance STD_tp_nom PAYE_months PAYE_tp_nom PAYE_tp ///
		REPAYE_Undergrad_months REPAYE_Undergrad_tp_nom REPAYE_Undergrad_tp ///
		HEA_months HEA_tp_nom HEA_tp
		
	export excel "${output}idr_analysis_results_list.xlsx", replace 
	
	

	
	
